@@ -1,4 +1,4 @@
-# Copyright (C) 2017-2023 Pier Carlo Chiodi
+# Copyright (C) 2017-2024 Pier Carlo Chiodi
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@ import sys
 import time
 import yaml
 import copy
+import re
 
 from jinja2 import Environment, FileSystemLoader, StrictUndefined
 
@@ -764,6 +765,11 @@ class ConfigBuilder(object):
                 return version.parse(self.target_version) <= version.parse(v)
             return False
 
+        def target_version_lt(v):
+            if self.target_version:
+                return version.parse(self.target_version) < version.parse(v)
+            return False
+
         def get_normalized_rtt(v):
             if not v:
                 return 0
@@ -803,6 +809,7 @@ class ConfigBuilder(object):
         env.filters["include_local_file"] = include_local_file
         env.filters["target_version_ge"] = target_version_ge
         env.filters["target_version_le"] = target_version_le
+        env.filters["target_version_lt"] = target_version_lt
         env.filters["get_normalized_rtt"] = get_normalized_rtt
         env.filters["to_bgp_role"] = to_bgp_role
 
@@ -875,9 +882,9 @@ class BIRDConfigBuilder(ConfigBuilder):
 
     AVAILABLE_VERSION = ["1.6.3", "1.6.4", "1.6.6", "1.6.7", "1.6.8",
                          "2.0.7", "2.0.7+b962967e", "2.0.8", "2.0.9",
-                         "2.0.10", "2.0.11", "2.13",
+                         "2.0.10", "2.0.11", "2.13", "2.14",
                          "3.0"]
-    DEFAULT_VERSION = "2.13"
+    DEFAULT_VERSION = "2.14"
 
     def validate_bgpspeaker_specific_configuration(self):
         res = True
@@ -1051,7 +1058,7 @@ class OpenBGPDConfigBuilder(ConfigBuilder):
     LOCAL_FILES_BASE_DIR = "/etc/bgpd"
 
     AVAILABLE_VERSION = ["7.0", "7.1", "7.2", "7.3", "7.4", "7.5", "7.6", "7.7",
-                         "7.8", "8.0"]
+                         "7.8", "8.0", "8.3"]
     DEFAULT_VERSION = AVAILABLE_VERSION[-1]
 
     IGNORABLE_ISSUES = ConfigBuilder.IGNORABLE_ISSUES + \
@@ -1321,6 +1328,79 @@ class TemplateContextDumper(ConfigBuilder):
 
 class IRRASSetBuilder(ConfigBuilder):
 
+    @staticmethod
+    def _get_source_from_template_file(path):
+        with open(path, "r") as f:
+            for line in f.readlines():
+                if not line.strip():
+                    continue
+
+                if line.strip().startswith(("source:", "- source:")):
+                    return line.strip().split(":")[1].strip()
+
+        return None
+
+    @staticmethod
+    def _get_as_set_info(as_set):
+        v = as_set.strip()
+
+        # Removing "ipv4:" and "ipv6:".
+        pattern = re.compile("^(?:ipv4|ipv6):", flags=re.IGNORECASE)
+        v, number_of_subs_made = pattern.subn("", v)
+        if number_of_subs_made > 0:
+            v = v.strip()
+
+        # Catching AS-FOO@SOURCE
+        pattern = re.compile("^([^@]+)@(.+)$", flags=re.IGNORECASE)
+        match = pattern.match(v)
+        if match:
+            return match.group(2), match.group(1)
+
+        # Catching ASxxx:AS-FOO.
+        pattern = re.compile("^AS\\d+:[^:].+$", flags=re.IGNORECASE)
+        if pattern.match(v):
+            return None, v
+
+        # Catching "SOURCE:AS-FOO" (single colon)
+        pattern = re.compile("^([^:]+):([^:].+)$", flags=re.IGNORECASE)
+        match = pattern.match(v)
+        if match:
+            return match.group(1), match.group(2)
+
+        # Catching "SOURCE::AS-FOO" (double colon)
+        pattern = re.compile("^([^:]+)::([^:]?.+)$", flags=re.IGNORECASE)
+        match = pattern.match(v)
+        if match:
+            return match.group(1), match.group(2)
+
+        return None, v
+
+    def _get_valid_as_sets(self, original_list):
+        res = []
+        for raw_as_set in original_list:
+            source, as_set = self._get_as_set_info(raw_as_set)
+
+            if source:
+                if source == self._source:
+                    res.append(as_set)
+                else:
+                    logging.warning(
+                        f"AS-SET {raw_as_set} omitted because its source "
+                        f"({source}) does not match the source used in "
+                        f"the template ({self._source})"
+                    )
+            else:
+                res.append(as_set)
+
+        return res
+
+    def __init__(self, *args, **kwargs):
+        super(IRRASSetBuilder, self).__init__(*args, **kwargs)
+
+        self._source = self._get_source_from_template_file(self.template_path)
+        self._include_members = kwargs.get("include_members")
+        self._exclude_members = kwargs.get("exclude_members")
+
     def enrich_j2_environment(self, env):
 
         if self.ip_ver is None:
@@ -1353,7 +1433,7 @@ class IRRASSetBuilder(ConfigBuilder):
             # The client has its own list of AS-SETs.
             # Use it and move to the next client.
             if client_irrdb["as_sets"]:
-                members.update(client_irrdb["as_sets"])
+                members.update(self._get_valid_as_sets(client_irrdb["as_sets"]))
                 continue
 
             # If we're here, it means that the client
@@ -1367,7 +1447,7 @@ class IRRASSetBuilder(ConfigBuilder):
                 # The client ASN is configured in the 'asns'
                 # section of the clients.yml file and there
                 # are AS-SETs set on it. Let's use them.
-                members.update(self.cfg_asns.cfg["asns"][asn]["as_sets"])
+                members.update(self._get_valid_as_sets(self.cfg_asns.cfg["asns"][asn]["as_sets"]))
                 continue
 
             # Let's check if AS-SETs were found on PeeringDB.
@@ -1380,7 +1460,7 @@ class IRRASSetBuilder(ConfigBuilder):
                                     client["id"], client["asn"],
                                     ", ".join(as_sets_from_pdb)
                                 ))
-                members.update(as_sets_from_pdb)
+                members.update(self._get_valid_as_sets(as_sets_from_pdb))
                 continue
 
             # No other AS-SETs found for the client.
@@ -1389,4 +1469,14 @@ class IRRASSetBuilder(ConfigBuilder):
                                 client["id"], client["asn"]
                             ))
 
+        if self._include_members:
+            for member in self._include_members.split(","):
+                members.add(member.strip())
+
+        if self._exclude_members:
+            for member in self._exclude_members.split(","):
+                member = member.strip()
+                if member in members:
+                    members.remove(member)
+
         self.data["as_sets_rpsl_objects"] = sorted(members)
