@@ -1,4 +1,4 @@
-# Copyright (C) 2017-2023 Pier Carlo Chiodi
+# Copyright (C) 2017-2024 Pier Carlo Chiodi
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -365,6 +365,32 @@ class IRRASSetCommand(TemplateRenderingCommands):
     def _get_template_sub_dir(self):
         return "irr-as-set"
 
+    @classmethod
+    def add_arguments(cls, parser):
+        super(IRRASSetCommand, cls).add_arguments(parser)
+
+        parser.add_argument(
+            "--include-members",
+            help="Comma-delimited list of members that will be always added to "
+                 "the 'members' attribute of the object generated by this "
+                 "command.",
+            metavar="MEMBERS",
+            dest="include_members")
+
+        parser.add_argument(
+            "--exclude-members",
+            help="Comma-delimited list of members that will never be included "
+                 "in the 'members' attribute of the object generated by this "
+                 "command.",
+            metavar="MEMBERS",
+            dest="exclude_members")
+
+    def _set_cfg_builder_params(self):
+        super(IRRASSetCommand, self)._set_cfg_builder_params()
+
+        self.cfg_builder_params["include_members"] = self.args.include_members
+        self.cfg_builder_params["exclude_members"] = self.args.exclude_members
+
     def run(self):
         if not self.args.template_name:
             templates_dir = os.path.join(
