@@ -1,4 +1,4 @@
-# Copyright (C) 2017-2023 Pier Carlo Chiodi
+# Copyright (C) 2017-2024 Pier Carlo Chiodi
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -106,7 +106,7 @@ class RegistroBRWhoisDBDump(CachedObject):
             url = self.source
             try:
                 response = requests.get(url)
-                raw_data = response.text
+                raw_data = response.raw.read()
             except Exception as e:
                 raise RegistroBRWhoisDBDumpError(
                     "Error while retrieving Registro.br Whois DB dump "
