@@ -1,4 +1,4 @@
-# Copyright (C) 2017-2023 Pier Carlo Chiodi
+# Copyright (C) 2017-2024 Pier Carlo Chiodi
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -661,5 +661,15 @@ class OpenBGPD80PortableInstance(OpenBGPDPortableInstance):
     TARGET_VERSION = "8.0"
 
 
-OpenBGPDPortablePreviousInstance = OpenBGPD78PortableInstance
-OpenBGPDPortableLatestInstance = OpenBGPD80PortableInstance
+class OpenBGPD83PortableInstance(OpenBGPDPortableInstance):
+
+    DOCKER_IMAGE = "pierky/openbgpd:8.3"
+
+    TAG = "openbgpd83p"
+
+    BGP_SPEAKER_VERSION = "8.3"
+    TARGET_VERSION = "8.3"
+
+
+OpenBGPDPortablePreviousInstance = OpenBGPD80PortableInstance
+OpenBGPDPortableLatestInstance = OpenBGPD83PortableInstance
